/*: TAVL_RMV.C	TAVL Tree library    Version 3.01   6-feb-93
 *: Prototype:	void tavl_remove(TAVL_treeptr tree, TAVL_nodeptr *p)
 *: Purpose:	Removes from threaded AVL tree the node "p".  "p"
 *.		must point to a node which belongs to the tree - so
 *.		"p" must have been obtained via functions tavl_insert(),
 *.		tavl_succ(), tavl_pred(), or tavl_find().
 *
 *  RELEASED TO THE PUBLIC DOMAIN
 *
 *		 author:    Bert C. Hughes
 *			    200 N.Saratoga
 *			    St.Paul, MN 55104
 *			    Compuserve 71211,577
 */

#include <stdlib.h>   /* for prototype of abs() */
#include "tavlpriv.h"

void tavl_remove(TAVL_treeptr tree, TAVL_nodeptr p)
{
     TAVL_nodeptr q;
     int side;
     char deltaht = 1;

     if (!p || Is_Head(p)) return;

     (*tree->free_item)(p->dataptr);

     for (;;) {
	  if (RLINK(p)) {
	       q = minOf(p->Rptr);
	       p->dataptr = q->dataptr;
	  }
	  else if (LLINK(p)) {
	       q = maxOf(p->Lptr);
	       p->dataptr = q->dataptr;
	  }
	  else break;	/* now p points to a leaf */

	  p = q;
     }			/* until p points to a leaf */

     q = parentOf(p);
     if (p->rchild) {
	  q->Rptr = p->Rptr;
	  q->Rbit = THREAD;
	  side = RIGHT;
     }
     else {
	  q->Lptr = p->Lptr;
	  q->Lbit = THREAD;
	  side = LEFT;
     }

     (*tree->dealloc)(p);

     while (deltaht && !Is_Head(q)) {
	  q->bf -= side;
	  side = (q->rchild ? RIGHT : LEFT);
	  if (abs(q->bf) > 1)
	       q = rebalance_tavl(q,&deltaht);
	  else
	       deltaht = (q->bf == 0);	/* height longest side shrank */
	  q = parentOf(q);		/* to equal the shorter side  */
     }
} /* tavl_remove */
